import { isFunction, isObeject } from "@/utils/types.js"
import Toast from "@/lib/toast/index.js"
import store from "@/store/index.js"
import { baseUrl } from '@/config/index.js'
/**
 * @desc 倒计时
 * @param { Number } num 倒计时时间
 * @param { Functiom } cb 回调 
 */
export function timerFn(num = 60, cb) {
	let timer = setInterval(() => {
		cb && isFunction(cb) && cb(num)
		num --
		if(num < 0) {
			clearInterval(timer)
			timer = null
			return
		}
	}, 1000)
}

/**
 * @desc 节流函数 cv的
 * @param {Function} callback 回调函数
 * @param {Number} wait 间隔时间
 * @return {Function} 节流函数
 */
export function throttle(callback, wait = 3000) {
    let timer = null;
    let startTime;
    return function () {
        const ctx = this;
        const args = arguments;
        const now = +new Date();
        if (startTime && now < startTime + wait) {
            clearTimeout(timer);
            timer = setTimeout(function () {
                startTime = now;
                callback.apply(ctx, args);
            }, wait);
        } else {
            startTime = now;
            callback.apply(ctx, args);
        }
    }
}

/**
 * @desc 防抖函数  cv的
 * @param {需要防抖的函数} func
 * @param {延迟时间} wait
 * @param {是否立即执行} immediate
 */
export function debounce(func, wait = 1000, immediate = true) {
  let timeout
  return function(...args) {
    let context = this
    if (timeout) clearTimeout(timeout)

    if (immediate) {
      let callNow = !timeout
      timeout = setTimeout(function() {
        timeout = null
      }, wait)
      if (callNow) func.apply(context, args)
    } else {
      timeout = setTimeout(function() {
        func.apply(context, args)
      }, wait)
    }
  }
}

/**
 * 联系老板 
 */
export function makePhoneCall() {
	const storeInfo = store.state.user.storeInfo
	if(storeInfo && storeInfo.shop_tel) {
		const phoneNumber = storeInfo.shop_tel
		uni.showModal({
			content: `是否拨打${phoneNumber}`,
			success: res => {
				if(res.confirm) {
					uni.makePhoneCall({
						phoneNumber
					})
				} 
			}
		})
	}
}

export function getURLwithArgs(){
	const pages = getCurrentPages()
	const currentPage = pages[pages.length-1]
	const url = currentPage.route
	const options = currentPage.options
	//拼接url的参数
	let urlWithArgs = `${url}?`
	for(let key in options){
		const value = options[key]
		urlWithArgs += `${key}=${value}&`
	}
	urlWithArgs = urlWithArgs.substring(0, urlWithArgs.length-1)
	return urlWithArgs
}

/**
 * @desc 微信接口promise
 * @param { String } wxapi *
 * @param { Object  } ops 接口的配置
 * @param { Object } cops 自定义的配置   
 */
export function cWx(wxapi, ops = {}, cops = {}) {
	if(!isObeject(ops) || !isObeject(cops)) throw new Error('配置必须是对象')
	if(cops.loading) {
		wx.showLoading({
			mask: true
		})
	}
	return new Promise((resolve, reject) => {
		wx[wxapi]({
			...ops,
			success: res => {
				cops.scb && typeof cops.scb === 'function' && cops.scb()
				resolve({status: true, data: res, msg: 'success'})
			},
			fail: err => {
				cops.fcb && typeof cops.fcb === 'function' && cops.fcb()
				reject({status: true, data: null, msg: err})
			},
			complete: () => {
				cops.ccb && typeof cops.ccb === 'function' && cops.ccb()
				if(cops.loading) {
					wx.hideLoading()
				}
			}
		})
	})
}

// 初始化评论的数据格式  评论详情，评论中心和评论页面都用到这个方法  我日
export function resetCommentData(data) {
	if(isObeject(data)) {
		data = [{
			...data
		}]
	}
	const resetData = data.map(item => {
		let media = []
		let reply = null
		let add_data = null
		if(item.img_url && item.img_url !== 'null') {
			media = [...item.img_url.split(',').map(item => ({
				url: `${baseUrl}/${item}`,
				type: 'image'
			}))]
		}
		if(item.video_url && item.video_url !== 'null') {
			media.unshift({
				url: `${baseUrl}/${item.video_url}`,
				type: 'video'
			})
		}
		reply = item.reply ? item.reply.map(r => {
			const videos = r.video_url && {
				url: `${baseUrl}/${r.video_url}`,
				type: 'video'
			} || null
			const imgs = (r.img_url && r.img_url !== 'null' ) ? r.img_url.split(',').map(url => ({
				url: `${baseUrl}/${url}`,
				type: 'image'
			})) : []
			let r_media = []
			r_media = [...imgs]
			if(videos) {
				r_media.unshift(videos)
			}
			return {
				content: r.content,
				user_name: r.user_name,
				video_url: videos,
				img_url: imgs,
				// ri
				media: r_media
			}
		}) : null
		add_data = item.add_data ? { 
			content: item.add_data.content,
			past_day: item.past_day,
			video_url: item.add_data.video_url && {
				url: `${baseUrl}/${item.add_data.video_url}`,
				type: 'video'
			},
			reply: item.add_data.reply ? item.add_data.reply.map(r => {
				const videos = r.video_url && {
					url: `${baseUrl}/${r.video_url}`,
					type: 'video'
				} || null
				const imgs = (r.img_url && r.img_url !== 'null') ? r.img_url.split(',').map(url => ({
					url: `${baseUrl}/${url}`,
					type: 'image'
				})) : []
				let r_media = []
				r_media = [...imgs]
				if(videos) {
					r_media.unshift(videos)
				}
				return {
					content: r.content,
					user_name: r.user_name,
					video_url: videos,
					img_url: imgs,
					// ri
					media: r_media
				}
			}) : null,
			img_url: (item.add_data.img_url && item.add_data.img_url !== 'null') ? item.add_data.img_url.split(',').map(url => ({
				url: `${baseUrl}/${url}`,
				type: 'image'
			})) : []
		} : null
		return {
			user: {
				nickname: item.nickname,
				avatar: item.avatar,
				add_time: item.add_time,
				com_grade: item.com_grade,
				serve_grade: item.serve_grade || 0,
				goods_grade: item.goods_grade || 0,
				logistics_grade: item.logistics_grade || 0
			},
			goods: {
				goods_name: item.goods_name || null,
				goods_img: item.goods_img ? `${baseUrl}/${item.goods_img}` : null
			},
			content: item.content,
			media,
			id: item.id,
			reply,
			goods_attr: item.goods_attr,
			add_data
		}
	})
	return resetData
}